#!/usr/bin/env python

##system-logviewer
##
## Copyright (C) 2001-2004 Red Hat, Inc.
## Copyright (C) 2001-2004 Tammy Fox <tfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#Originally Written by Tammy Fox
#Last Modified May 1, 2003

## Coding conventions
## function names have underscores (_)
## variable names are semiStudlyCapped
## class names are StudlyCapped
## class instances are semiStudlyCapped

from gtk import *
import string
import gtk
import gobject
import os
import sys

import LogViewerGui
import LogBuffer

class LogFileClass:

    def __init__(self, key, prefs):
        self.key = key
        self.prefs = prefs
        self.buffer = LogBuffer.LogBuffer()
        self.searchBuffer = LogBuffer.LogBuffer()
        self.file_mtime = 0
        self.file_name = ''
        self.file_filter = ''
        self.log_file_handle = None

    def read_log(self):

        re_read = False
        do_append = False

        # grab latest info
        name = self.prefs.stringDict[self.key][0]
        location = self.prefs.logDict[name][1]
        filter = self.prefs.stringDict[self.key][1]
        # self.prefs.dump_values(self.key,'***LogFileClass::read_log***',self.prefs.fileListStore)

        # if filter changes re-read
        if filter != self.file_filter:
            self.file_filter = filter
            re_read = True

        # if file name changes re-read
        if location != self.file_name:
            re_read = True

        # if file is not open re-read
        if self.log_file_handle == None:
            re_read = True

        # if not re_read and modified, do append
        if re_read == False:
            do_append = self.file_mtime != os.stat(location)[8]

        if re_read:

            # if re_read, clear display, close file handle (if open)
            # open file, do all lines, set file_name and file_mtime
            try:
                #clear
                self.buffer.clear_buffer()
                self.buffer.set_text('')
                self.file_name = ''
                self.file_mtime = 0

                #close handle if open
                if self.log_file_handle != None:
                    self.log_file_handle.close()
                    self.log_file_handle = None

                #open the file
                self.log_file_handle = open(location, "r")

                #append all the lines
                self.append_lines(self.log_file_handle.readlines())
                
                #update file name and mtime
                self.file_name = location
                self.file_mtime =  os.stat(location)[8]

            except:
                #log file problem
                if self.log_file_handle != None:
                    self.log_file_handle.close()
                    self.log_file_handle = None
                self.file_name = ''
                self.file_mtime = 0
                self.buffer.set_buffer_text('File read of ' + location + ' failed.')

        elif do_append:

            # if do_append, do new lines, set file_mtime 
            try:
                
                #append all the lines
                self.append_lines(self.log_file_handle.readlines())

                #update file mtime
                self.file_mtime =  os.stat(location)[8]

            except:
                #log file problem
                if self.log_file_handle != None:
                    self.log_file_handle.close()
                    self.log_file_handle = None
                self.file_name = ''
                self.file_mtime = 0
                self.buffer.set_buffer_text('File append from ' + location + ' failed.')

### Append lines to buffer

    def append_lines(self, new_lines):

        lines_in_buffer = self.buffer.size()
        iter = self.buffer.get_end_iter ()

        #store contents into messages and buffer
        for line in new_lines:

            # apply filtering
            do_insert = 1
            if self.file_filter != "":
                do_insert = 0
                if string.find(line, self.file_filter) > -1:
                    do_insert = 1
            
            # apply alerts
            if do_insert:
                alert = 0
                warning = 0
                if len(self.prefs.alertWords) > 0:
                    for word in self.prefs.alertWords:
                        if (string.find(line.lower(), word.lower()) > -1):
                            self.buffer.insert_pixbuf(iter, self.prefs.alertPixbuf)
                            alert = 1
                            break
                if alert == 0 and len(self.prefs.warningWords) > 0:
                    for word in self.prefs.warningWords:
                        if (string.find(line.lower(), word.lower()) > -1):
                            self.buffer.insert_pixbuf(iter, self.prefs.warningPixbuf)
                            warning = 1
                            break
                if alert == 0 and warning == 0:
                    self.buffer.insert_pixbuf(iter, self.prefs.blankPixbuf)
                    
                #insert log file line into buffer
                if self.buffer.insert_into_buffer_at_offset(iter, line) == 0:
                    return

## Get status of log file

    def file_changed(self):
        # True = file exists and has changed
        try:
            #has file location changed?
            name = self.prefs.stringDict[self.key][0]
            fileName = self.prefs.logDict[name][1]
            if self.file_name != fileName:
                self.file_name = fileName
                return True
            #compare st_mtime from stat to previous st_mtime from current stat
            status = (os.stat(fileName))
            if (status[8] != self.file_mtime):
                return True
            else:
                return False
        except:
            self.buffer.set_buffer_text('File error accessing : ' + self.file_name)
            return False

## Refresh displayed content of log file
                    
    def auto_refresh(self,force_refresh):
        #if file changed reload
        if force_refresh or self.file_changed(): 
            self.read_log()
            return True
        return False

## Apply new filter

    def filter_display(self, filter):

        if filter.strip() != self.file_filter:
            self.prefs.stringDict[self.key][1] = filter.strip()
            self.read_log()
            return True
        return False
