#!/usr/bin/env python

##system-logviewer
##
## Copyright (C) 2001-2004 Red Hat, Inc.
## Copyright (C) 2001-2004 Tammy Fox <tfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#Originally Written by Tammy Fox
#Created June 22, 2002
#Last Modified May 1, 2003

## Coding conventions
## function names have underscores (_)
## variable names are semiStudlyCapped
## class names are StudlyCapped
## class instances are semiStudlyCapped

from gtk import *
import string
import gtk
import gobject
import os

import LogViewerGui
import LogBuffer
import LogFileClass

class LogFileFrame:

    def __init__(self, key, parent):
        self.parent = parent
        self.key = key
        self.prefs = parent.prefs
        self.filter = parent.prefs.stringDict[self.key][1]
        self.desc = parent.prefs.stringDict[self.key][2]
        self.logClassInstance = LogFileClass.LogFileClass(key,parent.prefs)

        #--------Create Frame -------------------------------------
        
        self.logVBox = gtk.VBox(False, 6)
        self.logVBox.set_border_width(6)
        self.logFrame = gtk.Frame(self.prefs.stringDict[self.key][0])
        self.logFrame.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        self.logFrame.set_border_width(6)        

        self.logFrame.add(self.logVBox)

        self.descriptionHBox = gtk.HBox(False, 6)
        self.descriptionHBox.set_border_width(2)

        image = gtk.Image()
        image.set_from_pixbuf(self.prefs.system_logviewerPixbuf)

        self.descriptionHBox.pack_start(image, False)
        self.desc_label = gtk.Label(self.desc)
        self.desc_label.set_line_wrap(True)
        self.descriptionHBox.pack_start(self.desc_label, False)

        self.logVBox.pack_start(self.descriptionHBox, False)

        self.logScrollwindow = gtk.ScrolledWindow()
        self.logScrollwindow.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        self.logScrollwindow.set_shadow_type (SHADOW_ETCHED_IN)
        
        self.logView = gtk.TextView()

        #populate buffer with buffer from logClassInstance
        self.logView.set_buffer(self.logClassInstance.buffer.get_buffer())
           
        self.logView.set_editable(False)
        self.logView.set_cursor_visible(False)
        
        self.logScrollwindow.add(self.logView)
        self.logVBox.pack_start(self.logScrollwindow)        

        self.searchHBox = gtk.HBox(False, 6)
        self.searchLabel = gtk.Label('Filter for: ')
        self.searchHBox.pack_start(self.searchLabel, False)
        self.searchEntry = gtk.Entry()
        self.searchEntry.set_text(self.filter)
        self.searchEntry.connect ('activate', self.on_search_clicked)
        self.searchHBox.pack_start(self.searchEntry, True, True)
        self.searchButton = self.button_with_pixbuf(self.prefs.search_iconPixbuf, 'Filter')
        self.searchButton.connect('clicked', self.on_search_clicked)
        self.searchHBox.pack_start(self.searchButton, False)        
        self.logVBox.pack_start(self.searchHBox, False)        

        self.resetButton = self.button_with_stock(gtk.STOCK_REFRESH, 'Reset')
        self.resetButton.connect('clicked', self.on_reset_clicked)
        self.searchHBox.pack_start(self.resetButton, False)        

    def get_frame(self):
        return self.logFrame

    def set_buffer(self, buffer):
        self.logView.set_buffer(buffer)

    def button_with_pixbuf (self, icon, labeltext):
        button = gtk.Button ()
        label = gtk.Label (labeltext)
        hbox = gtk.HBox (False, 2)
        align = gtk.Alignment (0.5, 0.5, 0.0, 0.0)

        image = gtk.Image()
        image.set_from_pixbuf(icon)        
        hbox.pack_start (image)
        
        hbox.pack_end (label)
        button.add (align)
        align.add (hbox)
        button.show_all ()
        return button

    def button_with_stock (self, stock_icon, labeltext):
        button = gtk.Button ()
        label = gtk.Label (labeltext)
        hbox = gtk.HBox (False, 2)
        align = gtk.Alignment (0.5, 0.5, 0.0, 0.0)
        image = gtk.Image()
        if stock_icon:
            image.set_from_stock(stock_icon, 2)
            hbox.pack_start (image)
        
        hbox.pack_end (label)
        button.add (align)
        align.add (hbox)
        button.show_all ()
        return button

## Search button callback

    def on_search_clicked(self, *args):

        searchString = self.searchEntry.get_text().strip()
        changed = self.logClassInstance.filter_display(searchString)
        if changed:
            self.scroll_to_end()
            LogViewerGui.set_status('Search results for ' + searchString + ' displayed.')

    def on_reset_clicked(self, *args):
        LogViewerGui.set_status('Reseting log file...')
        
        self.logClassInstance.auto_refresh(True)
        self.scroll_to_end()

        LogViewerGui.set_status('Contents reset.')

## Scroll to end of view

    def scroll_to_end(self):

       self.logView.scroll_mark_onscreen(self.logClassInstance.buffer.get_end_mark()) 

## Refresh class

    def auto_refresh(self,flag):
        return self.logClassInstance.auto_refresh(flag)

## Update for edit operations
   
    def update_gui(self):
        self.filter = self.prefs.stringDict[self.key][1]
        self.searchEntry.set_text(self.filter)
        self.desc = self.prefs.stringDict[self.key][2]
        self.desc_label.set_text(self.desc)
 
