#!/usr/bin/env python

##system-logviewer
##
## Copyright (C) 2001-2004 Red Hat, Inc.
## Copyright (C) 2001-2004 Tammy Fox <tfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#Originally Written by Tammy Fox
#Created December 12, 2001
#Last Modified January 23, 2003

## Coding conventions
## function names have underscores (_)
## variable names are semiStudlyCapped
## class names are StudlyCapped
## class instances are semiStudlyCapped

from gtk import *
import string
import sys
import gtk
import signal
import gobject
import os

import Preferences
import PrefsGui
import LogFileClass
import LogFileFrame
import LogBuffer

##---------------------------------------------------------------

##
## status bar set by multiple classes
##

statusBar = gtk.Statusbar()

##
## Set Statusbar function
##

def set_status(text):
    statusBar.pop(0)
    statusBar.push(0, text)

##---------------------------------------------------------------
##---------------------------------------------------------------

##
## Main Class
##

class LogViewerGui:

## Menu layout
    ui_mutable = '''<ui>
    <menubar name="MenuBar">
      <menu action="File">
        <menuitem action="Refresh Now"/>
        <menuitem action="Quit"/>
      </menu>
      <menu action="Edit">
        <menuitem action="Preferences"/>
      </menu>
      <menu action="Help">
        <menuitem action="Contents"/>
        <menuitem action="About"/>
      </menu>
    </menubar>
    </ui>'''
    ui_read_only = '''<ui>
    <menubar name="MenuBar">
      <menu action="File">
        <menuitem action="Refresh Now"/>
        <menuitem action="Quit"/>
      </menu>
      <menu action="Help">
        <menuitem action="Contents"/>
        <menuitem action="About"/>
      </menu>
    </menubar>
    </ui>'''

    ## Init
    
    def __init__ (self,width,height,allowConfigChanges,configFile,initialView):

        self.configFile = configFile
        self.prefsClass = None

        ##
        ## ListStore : name, key, location, filter, description, altered (-1=Deleted,0=No Change,1=Modified), notebook page number
        ## (location must be accessible to be in this list)
        self.fileListStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_INT, gobject.TYPE_INT)

        #--------Create GTK Window----------------------------------
        window = gtk.Window()
        self.toplevel = window
        window.connect ('destroy', lambda w: gtk.main_quit())
        window.set_title('System Logs')
        window.set_default_size(width, height)
        window.set_position(gtk.WIN_POS_CENTER)
        self.main_vbox = gtk.VBox(False, 0)
        window.add(self.main_vbox)

        #--------Notebook that displays log files and search box----
        self.logFileNotebook = gtk.Notebook()
        self.logFileNotebook.set_show_tabs(False)

        #so the comment string for the desktop file will be translated
	commentString = 'Examine system log files'

        #--------UIManager-----------------------------------------
	uimanager = gtk.UIManager()

        #--------Menuing-------------------------------------------
        action_group = gtk.ActionGroup('LogViewerGui')
        window.actiongroup = action_group
        accel_group = uimanager.get_accel_group()
        window.add_accel_group(accel_group)

        action_group.add_actions([
                                  ('File', None, '_File'),
                                  ('Help', None, '_Help'),
                                  ('Refresh Now', gtk.STOCK_REFRESH, '_Refresh Now', '<control>R', 'Refresh Log View', self.btn_refresh_cb),
                                  ('Quit', gtk.STOCK_QUIT, '_Quit', '<control>Q', 'Quit this application', self.quit_cb),
                                  ('Contents', gtk.STOCK_HELP, '_Contents', 'F1', 'Application help', self.help_cb),
                                  ('About', gtk.STOCK_QUIT, '_About', '<control>A', 'About this application', self.about_cb)])
        
        if allowConfigChanges == 1:
            action_group.add_actions([
                ('Edit', None, '_Edit'),
                ('Preferences', gtk.STOCK_PREFERENCES, '_Preferences', None, None, self.prefs_cb)])
           
        uimanager.insert_action_group(action_group,0)

        if allowConfigChanges == 1:
            uimanager.add_ui_from_string(self.ui_mutable)
        else:
            uimanager.add_ui_from_string(self.ui_read_only)

        menubar = uimanager.get_widget('/MenuBar')
        self.main_vbox.pack_start(menubar, False)

        #--------left list store of log file types------------------
        self.main_hpane = gtk.HPaned()
        self.main_vbox.pack_start(self.main_hpane)

        #--------status bar-----------------------------------------
        self.main_vbox.pack_start(statusBar, False)

        #--------create configuration class-------------------------
        self.prefs = Preferences.Preferences(self)

        window.set_icon(self.prefs.system_logviewerPixbuf)
        col = gtk.TreeViewColumn(None, gtk.CellRendererText(), text=0)
        self.logFileView = gtk.TreeView(self.fileListStore);
        self.logFileView.set_headers_visible(False)
        self.logFileView.append_column(col)
        self.logFileView.set_border_width(8)

        logFileViewScrollWindow = gtk.ScrolledWindow()
        logFileViewScrollWindow.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        logFileViewScrollWindow.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        logFileViewScrollWindow.set_size_request(width/4,height)
        logFileViewScrollWindow.add(self.logFileView)

        #--------add frame instances to notebook and treeview-------        
        self.main_hpane.add1(logFileViewScrollWindow)
        self.main_hpane.add2(self.logFileNotebook)
        names_list = self.prefs.logDict.keys()
        names_list.sort()
        for name in names_list:
            self.append_to_gui(name)
        
        selection = self.logFileView.get_selection()
        selection.connect('changed', self.select_log)

        #--------show gui---------------------------------------------
        window.show_all()

        #--------select the initial view------------------------
        iter = self.fileListStore.get_iter_root()
        viewToSet = iter
         
        while (iter != None):
            if (initialView == self.fileListStore.get_value(iter, 0)):
                viewToSet = iter
                break
            iter = self.fileListStore.iter_next(iter)
 
        if (viewToSet != None):
 	    selection.select_iter(viewToSet)
            self.logFileView.set_cursor(self.fileListStore.get_path(viewToSet), None, False)
  
        #--------keep track of last refresh rate pref-----------------
        self.lastRefreshRate = self.prefs.refreshRate

        #--------continual updates------------------------------------
        gobject.timeout_add(self.lastRefreshRate*1000, self.timer_refresh_cb)

        #--------mainloop---------------------------------------------
        main ()


## About Menu Item callback

    def about_cb(self, b):
        dlg = gtk.MessageDialog (None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK,
                                 'Log Viewer 0.9.13\n Copyright (c) 2002-2004 Red Hat, Inc.\n Copyright (c) 2002-2004 Tammy Fox <tfox@redhat.com>\n A graphical interface for watching log files')
        dlg.set_title('About Log Viewer')
        dlg.set_default_size(100, 100)
        dlg.set_position (gtk.WIN_POS_CENTER_ON_PARENT)
        dlg.set_border_width(2)
        dlg.set_modal(True)
        dlg.set_transient_for(self.toplevel)
        dlg.set_icon(self.prefs.system_logviewerPixbuf)
        rc = dlg.run()
        dlg.destroy()

## Help Menu Item callback

    def help_cb(self, b):
        help_page = self.prefs.help_page
        path = "/usr/bin/htmlview"

        if path == None:
            dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK,
                                    ('Help is not available.'))
            dlg.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
            dlg.run()
            dlg.destroy()
            return
		
        pid = os.fork()
        if not pid:
            os.execv(path, [path, help_page])

## Select Item from List Store callback

    def select_log(self, object):
        list, iter = object.get_selected()
        # set notebook page to selected log file
        try:
            # self.prefs.dump_values(list.get_value(iter, 1),'***log selected***',list)
            self.logFileNotebook.set_current_page(list.get_value(iter, 6))
            if self.prefs.logDict[list.get_value(iter, 0)][2].auto_refresh(False):
                self.prefs.logDict[list.get_value(iter, 0)][2].scroll_to_end()
        except:
            #notebook might not contain the page anymore if it is deleted via preferences
            pass
        return

## Preferences Menu Item callback

    def prefs_cb(self, b):
        if(self.prefsClass == None):
            self.prefsClass = PrefsGui.PrefsGui(self)
            self.prefsDialog = self.prefsClass.launch()
            self.prefsDialog.set_transient_for(self.toplevel)

        self.prefsClass.clear_modified()
        if self.prefsDialog.run() == gtk.RESPONSE_CLOSE:
            if self.prefsClass.is_modified():
                if self.prefs.write_config_file() == False:
                    self.prefsClass.error_dialog('There was an error saving the preferences.')
            self.prefsDialog.hide()
        
## Quit Menu Item callback

    def quit_cb(self, b):
        gtk.main_quit()

## Button based refresh
    
    def btn_refresh_cb(self, b):
        set_status('Refreshing log file...')
        #get liststore iter to refresh correct page
        listStore, listStoreIter = self.logFileView.get_selection().get_selected()
        if listStoreIter != None:
            # get current page number
            pageNum = self.fileListStore.get_value(listStoreIter, 6)
            # get name such as "System Logs"
            logName = self.fileListStore.get_value(listStoreIter, 0)
            # get change code
            changeCode = self.fileListStore.get_value(listStoreIter, 5)
            scroll_flag = False
            if changeCode == 1:
                self.prefs.logDict[logName][2].update_gui()
                scroll_flag = self.prefs.logDict[logName][2].auto_refresh(True)
            else:
                if len(self.prefs.logDict[logName]) == 3:
                    scroll_flag = self.prefs.logDict[logName][2].auto_refresh(b!=0)
            if scroll_flag:
                self.prefs.logDict[logName][2].scroll_to_end()
            self.fileListStore.set_value(listStoreIter, 5, 0)
            
        #reset status bar after refresh
        set_status('')

## Timer based refresh 
        
    def timer_refresh_cb(self):
        #change timeout value was changed via preferences
        if self.prefs.refreshRate != self.lastRefreshRate:
            self.lastRefreshRate = self.prefs.refreshRate            
            gobject.timeout_add(self.lastRefreshRate*1000, self.timer_refresh_cb)
            return False
        #do the refresh
        self.btn_refresh_cb(0)
        #return True so gtk.time_out will keep going
        return True

## Append a new log def to the gui

    def append_to_gui(self,name):

	if len(self.prefs.logDict[name]) == 2 and self.file_readable(self.prefs.logDict[name][1]):
            key = self.prefs.logDict[name][0]
            location = self.prefs.logDict[name][1]
            logFrame = LogFileFrame.LogFileFrame(key,self)
            self.prefs.logDict[name] = self.prefs.logDict[name] + [logFrame]
            frame = self.prefs.logDict[name][2].get_frame()
            label =  gtk.Label('')
            iter = self.fileListStore.append()
            page = self.logFileNotebook.append_page(frame, label)
            self.prefs.set_liststore_values(self.fileListStore, iter, name, page)
            # self.prefs.dump_values(key,'***GUI liststore***',self.fileListStore)

## Is the file readable

    def file_readable(self,fileName):

        rv = False
        try:
            if os.access(fileName,os.F_OK):
               handle = open(fileName,"r")
               handle.close()
               rv = True
        except:
            rv = False

        return rv

## Update on PrefsGui change
 
    def update_gui(self,key,type):

        name = self.prefs.stringDict[key][0]

        # find iter for this key in gui treeview
        list = self.fileListStore
        iter = list.get_iter_root()
        treeIter = None
        while iter:
            if list.get_value(iter, 1) == key:
                treeIter = iter 
                break
            iter = list.iter_next(iter)

        # delete case
        if type == -1:
            # delete operation
            if treeIter!=None:
                selection = self.logFileView.get_selection() 
                (selList, selIter) = selection.get_selected()
                selKey = selList.get_value(selIter,1)
                del_page = list.get_value(treeIter,6)
                list.remove(treeIter)
                self.logFileNotebook.remove_page(del_page)
                if selKey == key:
                     selection.select_iter(selList.get_iter_root())
        else:
            # add or modify
            if treeIter==None:
                # add
                self.append_to_gui(name)
                self.logFileNotebook.show_all()
            else:
                # modify
                self.prefs.set_liststore_values(list, treeIter, name, list.get_value(treeIter,6))
                list.set_value(treeIter,5,1)
