#!/usr/bin/env python

##system-logviewer-mcs - preferences file object
##
## Copyright (C) 2001-2004 Red Hat, Inc.
## Copyright (C) 2001-2004 Tammy Fox <tfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#Originally Written by Tammy Fox
#Last Modified May 6, 2003

## Coding conventions
## function names have underscores (_)
## variable names are semiStudlyCapped
## class names are StudlyCapped
## class instances are semiStudlyCapped

from gtk import *
import string
import gtk
import signal
import gobject
import os
import sys
import time
import LogFileClass
import LogFileFrame

##---------------------------------------------------------------

##---------------------------------------------------------------
##---------------------------------------------------------------

##
## Main Class
##

class Preferences:

    alertWords = list()
    warningWords = []

    #stringDict is { key: [name, filter, description] }
    # key is logdef (LOGDEFN)
    # stringDict[LOGDEF3] = "System Logs", error, "A System Log view with a default filter of error"
    stringDict = {}

    #logDict is { key: [logdef, location, Instance of LogFileFrame] }
    # key is log name, logdef is the key into the stringDict (LOGDEFN)
    # logDict["System Logs"] = LOGDEF3,/var/log/messages,Instance of class LogFileFrame(oxdeadbeef)
    logDict = {}

## Init

    def __init__ (self, parentClass):
        self.parentClass = parentClass
        self.configFile = parentClass.configFile
        ##
        ## ListStore : name, key, location, filter, description, altered (-1=Deleted,0=No Change,1=Modified), notebook page number
        ##
        self.fileListStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_INT, gobject.TYPE_INT)

        self.blankPixbuf = gtk.gdk.pixbuf_new_from_file('/usr/share/system-logviewer-mcs/pixmaps/blank.png')
        self.alertPixbuf = gtk.gdk.pixbuf_new_from_file('/usr/share/system-logviewer-mcs/pixmaps/alert.png')
        self.warningPixbuf = gtk.gdk.pixbuf_new_from_file('/usr/share/system-logviewer-mcs/pixmaps/warning.png')
        self.search_iconPixbuf = gtk.gdk.pixbuf_new_from_file('/usr/share/system-logviewer-mcs/pixmaps/search-icon.png')
        self.system_logviewerPixbuf = gtk.gdk.pixbuf_new_from_file('/usr/share/pixmaps/system-logviewer-mcs.png')
        self.help_page = "file:///usr/share/doc/system-logviewer-mcs-" + "0.9.13" + "/index.html"
        self.alert_bigPixbuf = gtk.gdk.pixbuf_new_from_file('/usr/share/system-logviewer-mcs/pixmaps/alert-big.png')
        self.warning_bigPixbuf = gtk.gdk.pixbuf_new_from_file('/usr/share/system-logviewer-mcs/pixmaps/warning-big.png')

        self.refreshRate = 2
        self.logDefsCount = 0
	self.read_config_file()

	# fill the gui
        name_list = self.logDict.keys()
        name_list.sort()
        for name in name_list:
             iter = self.fileListStore.append()
             self.set_liststore_values(self.fileListStore, iter, name, 0)

    #
    # add a new location to the dictionary (called by prefsGui, add function)
    #
    def add_location(self, va):

        #create unique key name
        key = "LOGDEF%s"%(self.logDefsCount)

        #bump LOGDEF number
        self.logDefsCount = self.logDefsCount + 1

        #add to stringDict
        self.stringDict[key] = [va[0], va[2], va[3]]

        #add to logDict
        self.logDict[va[0]] = [key, va[1]]

	return key

    #
    # Read in preferences file
    #
    def read_config_file(self):

        try:
            prefsFile = open(self.configFile, "r")
            prefs = prefsFile.readlines()
            prefsFile.close()
        except:
            noPrefsDialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, 'Preferences file not found.')
            noPrefsDialog.set_default_size(100,100)
            noPrefsDialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
            noPrefsDialog.set_border_width(2)
            noPrefsDialog.set_icon(self.system-logviewerPixbuf)
            rc = noPrefsDialog.run()
            noPrefsDialog.destroy()
            sys.exit(2)

        for line in prefs:
            line = string.strip(line)
            if line != "" and line[0]!="#":
                preference, values = string.split(line,"=")
                preference = string.strip(preference)
                if preference == "REFRESHRATE":
                    refreshRateString = string.strip(values)
                    self.refreshRate = int(refreshRateString)
                elif preference == "ALERTWORDS":
                    for word in string.split(values,","):
                        self.alertWords.append(word)
                elif preference == "WARNINGWORDS":
                    for word in string.split(values,","):
                        self.warningWords.append(word)
                #user-defined log files
                elif preference == "LOGDEF":
                    #unique key for user-defined log file
                    #CSV entries : name,location,filter,desc
                    preference = "LOGDEF%s"%(self.logDefsCount)
                    preference = string.strip(preference)
                    #add to stringDict
                    name = string.split(values,",")[0]
                    self.stringDict[preference] = [name,string.split(values,",")[2],string.split(values,",")[3]]
                    #add location to logDict
                    self.logDict[name] = [preference, string.split(values,",")[1]]
                    #bump logDefsCount
                    self.logDefsCount = self.logDefsCount + 1

        # Don't refresh faster than .5 hz
        if self.refreshRate < 2:
            self.refreshRate = 2

## Set liststore values from Dicts

    def set_liststore_values(self, list, iter, name, pgNum):
        # item 0 is the short name "System Logs"
        # item 1 is the key "LOGDEF3"
        # item 2 is the location "/var/log/messages"
        # item 3 is the filter "ERROR"
        # item 4 is the description "System log filtered for errors"
        # item 5 is the location change flag (-1=Deleted,0=No Change,1=Modified)
        # item 6 is the page number
        key = self.logDict[name][0]
        loc = self.logDict[name][1]
        list.set_value(iter, 0, name)
        list.set_value(iter, 1, key)
        list.set_value(iter, 2, loc)
        list.set_value(iter, 3, self.stringDict[key][1])
        list.set_value(iter, 4, self.stringDict[key][2])
        list.set_value(iter, 5, 0)
        list.set_value(iter, 6, pgNum)

## Write preferences out

    def write_config_file(self):

        #read in existing values
        try:
            configFile = open(self.configFile, "r")
            configLines = configFile.readlines()
            configFile.close()
        except:
            self.error_dialog('Cannot read configuration file.')
            #don't go through rest of function
            return

        #backup file
        try:
            os.rename(self.configFile, self.configFile + ".bak")
        except:
            self.error_dialog('Cannot backup configuration file.')
            #don't go through rest of function
            return

        #scan all lines, pass comments, replace keywords with our own
        alerts_found = False
        warnings_found = False
        refresh_rate_found = False
        logdefs_found = False
        pos = 0
        rateStr = "REFRESHRATE="
        rateStr = rateStr + str(self.refreshRate)
        for line in configLines:

            #skip comments marked with #
            line = string.strip(line)
            if line != "" and line[0]!="#":

                key, value = string.split(line,"=")
                key = string.strip(key)

                if key == "ALERTWORDS":
                    configLines[pos] = self.get_aw_words(key,self.alertWords)
                    alertsFound = True
                elif key == "WARNINGWORDS":
                    configLines[pos] = self.get_aw_words(key,self.warningWords)
                    warnings_found = True
                elif key == "REFRESHRATE":
                    configLines[pos] = rateStr
                    refresh_rate_found = True
                elif key == "LOGDEF":
                    configLines[pos] = "DELETETHIS"

            pos = pos + 1

        #end of else statement for updating preferences

	# add anything missing
        if alertsFound == False:
            configLines = configLines + [self.get_aw_words(key, self.alertWords)]
        if warnings_found == False:
            configLines = configLines + [self.get_aw_words(key, self.warningWords)]
        if refresh_rate_found == False:
            configLines = configLines + [rateStr]

        #add logdefs
        logdefs = self.get_logdef_words()
        for logdef in logdefs:
            configLines = configLines + [logdef]

        #write prefs back to file
        try:
            prefsFile = open(self.configFile, "w")
            for line in configLines:
                line = string.strip(line)
                if line != "DELETETHIS":
                    prefsFile.write(line+"\n")
            prefsFile.close()
        except:
            return False

        return True
        
#
# Def words builder for warnings and alerts
#

    def get_aw_words(self, key, words):

        retStr = key + "="
        length = len(words)
        for word in words:
            if length == 1:
                #if last word, don't add comma to end
                retStr = retStr + word
            else:
                retStr = retStr + word + ","
            length = length - 1
        retStr = retStr + "\n"

        return retStr

#
# Build the logDef lines
#

    def get_logdef_words(self):

        #grab all the names
        lines = []
        names = self.logDict.keys()
        for name in names:
            key = self.logDict[name][0]
            loc = self.logDict[name][1]
            strs = self.stringDict[key]
            logDef = "LOGDEF=" + name + ","
            logDef = logDef + loc + ","
            logDef = logDef + strs[1] + ","
            logDef = logDef + strs[2] + "\n"
            lines = lines + [logDef]
        return lines

# 
# Edit the location (from PrefsGui) 
#
    def edit_location(self,key,values):
        name = values[0]
        loc = values[1]
        filter = values[2]
        desc = values[3]
        old_name = ""
        # tree first
        list = self.fileListStore 
        iter = list.get_iter_root()
        found = False
        while iter:
            if list.get_value(iter, 1) == key:
                #modify with new values and flag
                old_name = list.get_value(iter,0)
                list.set_value(iter,0,name)
                list.set_value(iter,2,loc)
                list.set_value(iter,3,filter)
                list.set_value(iter,4,desc)
                found = True
                break
            iter = self.fileListStore.iter_next(iter)
        # if found, the dicts
        if found:
            if name != old_name:
                self.logDict[name] = self.logDict[old_name]
                del self.logDict [old_name]
            self.logDict[name][1] = loc
            self.stringDict[key][0]=name
            self.stringDict[key][1]=filter
            self.stringDict[key][2]=desc
        return found

    def dump_values(self,key,msg,list):
        name = self.stringDict[key][0]
        print '%s, key : %s, name %s' % (msg,key,name)
        print '%s , %s' % ( self.stringDict[key], self.logDict[name] )
        iter = list.get_iter_root()
        n_iter = 0
        found = False
        while iter:
            pos = 0
            if list.get_value(iter,1) == key:
                found = True
                while pos < 7:
                    print 'list[%d][%d]=%s' % ( n_iter, pos, list.get_value(iter,pos) )
                    pos = pos + 1
                break
            n_iter = n_iter + 1
            iter = list.iter_next(iter)
        if found==False:
            print 'WARNING : key not found in list store!'
        print '***end***\n'
