#!/usr/bin/env python

##system-logviewer - preferences dialog box
##
## Copyright (C) 2001-2004 Red Hat, Inc.
## Copyright (C) 2001-2004 Tammy Fox <tfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#Originally Written by Tammy Fox
#Last Modified May 6, 2003

## Coding conventions
## function names have underscores (_)
## variable names are semiStudlyCapped
## class names are StudlyCapped
## class instances are semiStudlyCapped

from gtk import *
import string
import gtk
import signal
import gobject
import os
import time
import Preferences
import LogFileClass
import LogFileFrame

##---------------------------------------------------------------

##---------------------------------------------------------------
##---------------------------------------------------------------

##
## Main Class
##

class PrefsGui:

    def __init__ (self, parent):
        self.prefs = parent.prefs
        self.parent = parent
        self.icon = parent.prefs.system_logviewerPixbuf
        self.is_modded = False 
        self.preferences = gtk.Dialog('Preferences')
	self.preferences.set_size_request(750, 400)
	self.preferences.set_position (gtk.WIN_POS_CENTER_ON_PARENT)
	self.preferences.set_border_width(2)
	self.preferences.set_modal(True)
        self.preferences.set_position(gtk.WIN_POS_CENTER_ON_PARENT)

        self.preferences.set_icon(self.icon)

	#--------Preferences Notebook-------------------------------
 	self.prefsNotebook = gtk.Notebook()
 	self.prefsNotebook.set_show_tabs(True)		
 	self.preferences.vbox.pack_start(self.prefsNotebook, True, True)
	
	#--------Log Files and Refresh Rate page----------------------
        self.logFilesVBox = gtk.VBox(False, 6)
        self.logFilesVBox.set_border_width(6)
        self.logFilesFrame = gtk.Frame('Log Files')
        self.logFilesFrame.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        self.logFilesFrame.set_border_width(6)        

        self.lFdescriptionHBox = gtk.HBox(False, 6)
        self.lFdescriptionHBox.set_border_width(2)

        prefsImage = gtk.Image()
        prefsImage.set_from_stock(gtk.STOCK_PREFERENCES, 5)

        self.lFdescriptionHBox.pack_start(prefsImage, False)

        label = gtk.Label('Log file definitions for each log file displayed.')
        label.set_line_wrap(True)
        # label.set_size_request(250, -1)
        self.lFdescriptionHBox.pack_start(label, False)

        self.logFilesVBox.pack_start(self.lFdescriptionHBox, False, False)

        self.logFilesScrollWindow = gtk.ScrolledWindow()
        self.logFilesScrollWindow.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        self.logFilesScrollWindow.set_shadow_type(gtk.SHADOW_ETCHED_IN)

	self.prefsTreeView = gtk.TreeView(self.prefs.fileListStore)
        col = gtk.TreeViewColumn(None, gtk.CellRendererText(), text=0)
        col.set_title('Log Name')
        col.set_visible(True)
        col2 = gtk.TreeViewColumn(None, gtk.CellRendererText(), text=2)
        col2.set_title('Location')
        col2.set_visible(True)
        col3 = gtk.TreeViewColumn(None, gtk.CellRendererText(), text=3)
        col3.set_title('Filter')
        col3.set_visible(True)
        col4 = gtk.TreeViewColumn(None, gtk.CellRendererText(), text=4)
        col4.set_title('Description')
        col4.set_visible(True)
        self.prefsTreeView.set_headers_visible(True)
        self.prefsTreeView.append_column(col)
        self.prefsTreeView.append_column(col2)
        self.prefsTreeView.append_column(col3)
        self.prefsTreeView.append_column(col4)
        self.prefsTreeView.set_border_width(8)
        selection = self.prefsTreeView.get_selection()

        self.prefsTreeView.set_rules_hint(True)
        self.logFilesScrollWindow.add(self.prefsTreeView)

        #Location buttons
        self.logFilesLocationsVBoxButtons = gtk.VBox(False,3)
        self.editButton = gtk.Button('_Edit')
        self.editButton.connect ('clicked', self.on_edit_log_clicked)
        self.addButton = gtk.Button('_Add')
        self.addButton.connect ('clicked', self.on_add_log_clicked)
        self.deleteButton = gtk.Button('_Delete')
        self.deleteButton.connect ('clicked', self.on_del_log_clicked)
        self.logFilesLocationsVBoxButtons.pack_start(self.addButton, False, False)
        self.logFilesLocationsVBoxButtons.pack_start(self.editButton, False, False)
        self.logFilesLocationsVBoxButtons.pack_start(self.deleteButton, False, False)

        self.logFileLocationsHBox = gtk.HBox(False, 10)
        self.logFileLocationsHBox.pack_start(self.logFilesScrollWindow, True, True)
        self.logFileLocationsHBox.pack_start(self.logFilesLocationsVBoxButtons, False, False)

        self.logFilesVBox.pack_start(self.logFileLocationsHBox, True, True)        

        self.logFilesFrame.add(self.logFilesVBox)

        #Refresh Rate spinner
        self.refreshRateFrame = gtk.Frame('Refresh Rate')
        self.refreshRateFrame.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        self.refreshRateFrame.set_border_width(6)      
        self.refreshRateLabel = gtk.Label('Refresh log file every ')
        self.refreshRateSpin = gtk.SpinButton(gtk.Adjustment(self.prefs.refreshRate,2,600,1,1,0), 0, 0)
        self.refreshRateSpin.connect('value-changed', self.on_refresh_rate_changed)
        self.refreshRateSecLabel = gtk.Label('seconds.')
        self.refreshRateHBox = gtk.HBox(False,3)
        self.refreshRateHBox.set_border_width(2)
        self.refreshRateHBox.pack_start(self.refreshRateLabel, False, False)
        self.refreshRateHBox.pack_start(self.refreshRateSpin, False, False)
        self.refreshRateHBox.pack_start(self.refreshRateSecLabel, False, False)

        self.refreshRateFrame.add(self.refreshRateHBox)

        self.logFilesVBox = gtk.VBox(False,2)
        self.logFilesVBox.pack_start(self.logFilesFrame, True, True)
        self.logFilesVBox.pack_start(self.refreshRateFrame, False, False)
        label = gtk.Label('Log _Files')
        label.set_use_underline(True)
        self.prefsNotebook.append_page(self.logFilesVBox, label)
        

	#--------Alert Icon page---------------------------------------------
        self.alertsVBox = gtk.VBox(False, 2)
        self.alertsVBox.set_border_width(6)
        self.alertsFrame = gtk.Frame('Alerts')
        self.alertsFrame.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        self.alertsFrame.set_border_width(6)

        self.alertsDescriptionHBox = gtk.HBox(False, 6)
        self.alertsDescriptionHBox.set_border_width(2)

        image = gtk.Image()
        image.set_from_pixbuf(self.prefs.alert_bigPixbuf)
        self.alertsDescriptionHBox.pack_start(image, False)

        label = gtk.Label('Show alert icon for the following key words.')
        label.set_line_wrap(True)
        # label.set_size_request(250, -1)
        self.alertsDescriptionHBox.pack_start(label, False)

        self.alertsVBox.pack_start(self.alertsDescriptionHBox, False, False)

        #list store that contains key words to alert
        self.alertsListStore = gtk.ListStore(gobject.TYPE_STRING)
        for word in self.prefs.alertWords:
            #insert word into liststore
            if string.strip(word) != "":
                iter = self.alertsListStore.append()
                self.alertsListStore.set_value(iter, 0, string.strip(word))

	self.alertsTreeView = gtk.TreeView(self.alertsListStore)
        col = gtk.TreeViewColumn(None, gtk.CellRendererText(), text=0)
        col.set_visible(True)
        self.alertsTreeView.set_headers_visible(False)
        self.alertsTreeView.append_column(col)
        self.alertsTreeView.set_border_width(8)
        selection = self.alertsTreeView.get_selection()

        self.alertsHBox = gtk.HBox(False, 10)
        
        self.alertsTreeView.set_rules_hint(True)
        self.alertsScrollWindow = gtk.ScrolledWindow()
        self.alertsScrollWindow.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        self.alertsScrollWindow.set_shadow_type(gtk.SHADOW_ETCHED_IN)

        self.alertsScrollWindow.add(self.alertsTreeView)

        self.alertsHBox.pack_start(self.alertsScrollWindow, True, True)

        self.alertsButtonVBox = gtk.VBox(False,2)
        self.alertsAddButton = gtk.Button('_Add')
        self.alertsAddButton.connect ('clicked', self.on_add_alert)
        self.alertsDeleteButton = gtk.Button('_Delete')
        self.alertsDeleteButton.connect ('clicked', self.on_delete_alert)
        self.alertsButtonVBox.pack_start(self.alertsAddButton, False, False)
        self.alertsButtonVBox.pack_start(self.alertsDeleteButton, False, False)
        self.alertsHBox.pack_start(self.alertsButtonVBox, False, False)

        self.alertsVBox.pack_start(self.alertsHBox)
        self.alertsFrame.add(self.alertsVBox)
        label = gtk.Label('A_lerts')
        label.set_use_underline(True)
        self.prefsNotebook.append_page(self.alertsFrame, label)
        

	#--------Warning Icon page---------------------------------------------
        self.warningsVBox = gtk.VBox(False, 2)
        self.warningsVBox.set_border_width(6)
        self.warningsFrame = gtk.Frame('Warnings')
        self.warningsFrame.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        self.warningsFrame.set_border_width(6)

        self.warningsDescriptionHBox = gtk.HBox(False, 6)
        self.warningsDescriptionHBox.set_border_width(2)

        icon = self.prefs.warning_bigPixbuf
        image = gtk.Image()
        image.set_from_pixbuf(icon)

        self.warningsDescriptionHBox.pack_start(image, False)

        label = gtk.Label('Show warning icon for the following key words.')
        label.set_line_wrap(True)
        # label.set_size_request(250, -1)
        self.warningsDescriptionHBox.pack_start(label, False)

        self.warningsVBox.pack_start(self.warningsDescriptionHBox, False, False)

        #list store that contains key words to warning
        self.warningsListStore = gtk.ListStore(gobject.TYPE_STRING)
        for word in self.prefs.warningWords:
            #insert word into liststore
            if string.strip(word) != "":
                iter = self.warningsListStore.append()
                self.warningsListStore.set_value(iter, 0, string.strip(word))

	self.warningsTreeView = gtk.TreeView(self.warningsListStore)
        col = gtk.TreeViewColumn(None, gtk.CellRendererText(), text=0)
        col.set_visible(True)
        self.warningsTreeView.set_headers_visible(False)
        self.warningsTreeView.append_column(col)
        self.warningsTreeView.set_border_width(8)
        selection = self.warningsTreeView.get_selection()

        self.warningsHBox = gtk.HBox(False, 10)
        
        self.warningsTreeView.set_rules_hint(True)
        self.warningsScrollWindow = gtk.ScrolledWindow()
        self.warningsScrollWindow.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        self.warningsScrollWindow.set_shadow_type(gtk.SHADOW_ETCHED_IN)

        self.warningsScrollWindow.add(self.warningsTreeView)

        self.warningsHBox.pack_start(self.warningsScrollWindow, True, True)

        self.warningsButtonVBox = gtk.VBox(False,2)
        self.warningsAddButton = gtk.Button('_Add')
        self.warningsAddButton.connect ('clicked', self.on_add_warning)
        self.warningsDeleteButton = gtk.Button('_Delete')
        self.warningsDeleteButton.connect ('clicked', self.on_delete_warning)
        self.warningsButtonVBox.pack_start(self.warningsAddButton, False)
        self.warningsButtonVBox.pack_start(self.warningsDeleteButton, False)
        self.warningsHBox.pack_start(self.warningsButtonVBox, False)

        self.warningsVBox.pack_start(self.warningsHBox)
        self.warningsFrame.add(self.warningsVBox)
        label = gtk.Label('_Warnings')
        label.set_use_underline(True)
        self.prefsNotebook.append_page(self.warningsFrame, label)

        #--------Buttons-------------------------------------------
        self.preferences.add_button('gtk-close', gtk.RESPONSE_CLOSE)

        #--------Show Dialog---------------------------------------
        self.preferences.show_all()

## Edit Log File Location

    def on_edit_log_clicked(self, *args):

        #grab log file locations
        list, iter = self.prefsTreeView.get_selection().get_selected()
        if iter == None:
            self.error_dialog('Please select a log file from the list.')
            return

        values = ["","","",""]
        key = list.get_value(iter, 1)
        values[0] = list.get_value(iter, 0)
        values[1] = list.get_value(iter, 2)
        values[2] = list.get_value(iter, 3)
        values[3] = list.get_value(iter, 4)
        rc = self.run_logdef_dialog('Edit log definition','Edit existing log file definition.',values)
        if rc == 1:
            aName = string.strip(values[0])
            aPath = string.strip(values[1])
            if len(aName) > 0 and len(aPath) > 0:
                if self.prefs.edit_location(key, values) == True:
                    self.parent.update_gui(key,1)
                    self.is_modded = True 
            else:
                self.error_dialog('Name and Location must be set.')

## Add Log File Location

    def on_add_log_clicked(self, *args):
        values = ["","","",""]
        rc = self.run_logdef_dialog('Add log definition','Add a new log file definition.',values)
        if rc == 1:
            aName = string.strip(values[0])
            aPath = string.strip(values[1])
            if len(aName) > 0 and len(aPath) > 0:
                key = self.prefs.add_location(values)
                iter = self.prefs.fileListStore.append()
                self.prefs.set_liststore_values(self.prefs.fileListStore,iter,aName,0)
                self.parent.update_gui(key,0)
                self.is_modded = True 
            else:
                self.error_dialog('Name and Location must be set.')

## Add/Edit log definition dialog

    def run_logdef_dialog(self,title,tip,valueIo):
        aDialog = gtk.Dialog(title)
        aDialog.set_position (gtk.WIN_POS_CENTER_ON_PARENT)
        aDialog.set_border_width(2)
        aDialog.set_modal(True)
        aDialog.set_transient_for(self.preferences)
        aDialog.set_icon(self.icon)
        aDialog.add_button('gtk-cancel', gtk.RESPONSE_CANCEL)
        aDialog.add_button('gtk-ok', gtk.RESPONSE_OK)
        clLabel = gtk.Label(tip)
        clLabel.set_line_wrap(True)

        prefsImage = gtk.Image()
        prefsImage.set_from_stock(gtk.STOCK_PREFERENCES, 3)

        fileHBox = gtk.HBox(False, 6)
        fileHBox.pack_start(prefsImage, False)
        fileHBox.pack_start(clLabel, False)
        fileNameLabel = gtk.Label('Name:')
        fileNameLabel.set_alignment(1.0, 0.0)
        fileNameEntry = gtk.Entry()
        fileNameEntry.set_text(valueIo[0])
        fileLocationLabel = gtk.Label('Location:')
        fileLocationLabel.set_alignment(1.0, 0.0)
        fileLocationEntry = gtk.Entry()
        fileLocationEntry.set_text(valueIo[1])
        fileFilterLabel = gtk.Label('Filter:')
        fileFilterLabel.set_alignment(1.0, 0.0)
        fileFilterEntry = gtk.Entry()
        fileFilterEntry.set_text(valueIo[2])
        fileDescLabel = gtk.Label('Description:')
        fileDescLabel.set_alignment(1.0, 0.0)
        fileDescEntry = gtk.Entry()
        fileDescEntry.set_text(valueIo[3])

        fileTable = gtk.Table(rows=4,columns=2)
        fileTable.set_homogeneous(False)
        fileTable.set_row_spacings(3)
        fileTable.set_col_spacings(3)

        #attach(child,left_attach,right_attach,top_attach,bottom_attach)
        fileTable.attach(fileNameLabel, 0, 1, 0, 1)
        fileTable.attach(fileNameEntry, 1, 2, 0, 1)
        fileTable.attach(fileLocationLabel, 0, 1, 1, 2)
        fileTable.attach(fileLocationEntry, 1, 2, 1, 2)
        fileTable.attach(fileFilterLabel, 0, 1, 2, 3)
        fileTable.attach(fileFilterEntry, 1, 2, 2, 3)
        fileTable.attach(fileDescLabel, 0, 1, 3, 4)
        fileTable.attach(fileDescEntry, 1, 2, 3, 4)

        aDialog.vbox.pack_start(fileHBox, False)
        aDialog.vbox.pack_start(fileTable, False)
        aDialog.vbox.set_spacing(15)
        aDialog.show_all()
        rv = 0
        rc = aDialog.run()
        if rc == gtk.RESPONSE_OK:
            valueIo[0] = string.strip(fileNameEntry.get_text())
            valueIo[1] = string.strip(fileLocationEntry.get_text())
            valueIo[2] = string.strip(fileFilterEntry.get_text())
            valueIo[3] = string.strip(fileDescEntry.get_text())
            rv = 1
        aDialog.hide()
        return rv
        
## Called by LogViewerGui to show Preferences Dialog

    def launch(self):
        return self.preferences

## Error dialog

    def error_dialog (self, errorMsg):
        dialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, errorMsg)
        dialog.set_title = ('Error')
        dialog.set_default_size(100,100)
        dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        dialog.set_border_width(2)
        dialog.set_modal(True)
        dialog.set_transient_for(self.preferences)
        dialog.set_icon(self.icon)
        # dialog.set_has_separator(False)
        rc = dialog.run()
        if rc == gtk.RESPONSE_OK:
            dialog.hide()

## Add alert keyword

    def on_add_alert (self, *args):
        self.addAlertsDialog = gtk.Dialog('Add Alert Word')
        self.addAlertsDialog.set_default_size(100, 100)
        self.addAlertsDialog.set_position (gtk.WIN_POS_CENTER_ON_PARENT)
        self.addAlertsDialog.set_border_width(2)
        self.addAlertsDialog.set_modal(True)
        self.addAlertsDialog.set_transient_for(self.preferences)
        self.addAlertsDialog.set_icon(self.icon)
        self.addAlertsDialog.set_has_separator(False)
        label = gtk.Label('Enter a word to add to the list of alert words.')
        addAlertsVBox = gtk.VBox(False, 6)
        addAlertsVBox.pack_start(label, True, True, 6)
        self.alertsEntry = gtk.Entry()
        self.alertsEntry.connect ('activate', self.add_alert)

        alertsCancelButton = self.addAlertsDialog.add_button('gtk-cancel', 0)
        alertsCancelButton.connect ('clicked', self.on_cancel_alert)        
                                          
        alertsOKButton = self.addAlertsDialog.add_button('gtk-ok', 0)
        alertsOKButton.connect ('clicked', self.add_alert)
                                         
        addAlertsVBox.pack_start(self.alertsEntry, True, True, 6)

        self.addAlertsDialog.vbox.pack_start(addAlertsVBox, True)
        self.addAlertsDialog.show_all()

    def on_cancel_alert(self, *args):
        self.addAlertsDialog.destroy()
        
    def add_alert(self, *args):
        alert = string.strip(self.alertsEntry.get_text())
        if string.find(alert,',') > -1 or string.find(alert,'=') > -1:
            self.error_dialog('Alert word can not contain a comma (,) or equals sign (=).')
            return
        if alert != "":
            #add to liststore
            iter = self.alertsListStore.append()
            self.alertsListStore.set_value(iter, 0, alert)
            #add to prefs alertWords
            self.prefs.alertWords.append(alert)
            #hide dialog
            self.addAlertsDialog.hide()
            self.is_modded = True 
        else:
            self.error_dialog('Please specify an alert word to add.')
            return

## Delete alert keyword

    def on_delete_alert (self, *args):
        #grab selected alert word
        data, iter = self.alertsTreeView.get_selection().get_selected()
        if iter == None:
            self.error_dialog('Please select an alert word from the list.')
            return

        alert = data.get_value(iter, 0)
        #delete from ListStore to delete from TreeView
        data.remove(iter)
        #delete from alertWords
        self.prefs.alertWords.remove(alert)

## Delete Log File Location

    def on_del_log_clicked(self, *args):
        listStore, treeIter = self.prefsTreeView.get_selection().get_selected()
                        
        if treeIter == None:
            self.error_dialog('Please select a log file from the list.')
            return

        #remove from internal list store
        key = listStore.get_value(treeIter, 1)
        listStore.remove(treeIter)

        #remove from logviewer
        self.parent.update_gui(key,-1)

        self.is_modded = True

## Add warning keyword

    def on_add_warning (self, *args):
        self.addWarningDialog = gtk.Dialog('Add Warning Word')
        self.addWarningDialog.set_default_size(100, 100)
        self.addWarningDialog.set_position (gtk.WIN_POS_CENTER_ON_PARENT)
        self.addWarningDialog.set_border_width(2)
        self.addWarningDialog.set_modal(True)
        self.addWarningDialog.set_transient_for(self.preferences)
        self.addWarningDialog.set_icon(self.icon)
        self.addWarningDialog.set_has_separator(False)
        label = gtk.Label('Enter a word to add to the list of warning words.')
        addWarningsVBox = gtk.VBox(False, 6)
        addWarningsVBox.pack_start(label, True, True, 6)
        self.warningsEntry = gtk.Entry()
        self.warningsEntry.connect ('activate', self.add_warning)

        cancelButton = self.addWarningDialog.add_button('gtk-cancel', 0)
        cancelButton.connect ('clicked', self.on_cancel_warning)        
                                          
        OKButton = self.addWarningDialog.add_button('gtk-ok', 0)
        OKButton.connect ('clicked', self.add_warning)
                                         
        addWarningsVBox.pack_start(self.warningsEntry, True, True, 6)

        self.addWarningDialog.vbox.pack_start(addWarningsVBox, True)
        self.addWarningDialog.show_all()

    def on_cancel_warning(self, *args):
        self.addWarningDialog.destroy()
        
    def add_warning(self, *args):
        warning = string.strip(self.warningsEntry.get_text())
        if string.find(warning,',') > -1 or string.find(warning,'=') > -1:
            self.error_dialog('Warning word can not contain a comma (,) or equals sign (=).')
            return
        if warning != "":
            #add to prefs gui
            iter = self.warningsListStore.append()
            self.warningsListStore.set_value(iter, 0, warning)
            #add to prefs warningWords
            self.prefs.warningWords.append(warning)
            #hide dialog
            self.addWarningDialog.hide()
            self.is_modded = True 
        else:
            self.error_dialog('Please specify an warning word to add.')
            return

## Delete warning keyword

    def on_delete_warning (self, *args):
        #grab selected warning word
        data, iter = self.warningsTreeView.get_selection().get_selected()
        if iter == None:
            self.error_dialog('Please select an warning word from the list.')
            return

        warning = data.get_value(iter, 0)
        #delete from prefs gui
        data.remove(iter)
        #delete from prefs warningWords
        self.prefs.warningWords.remove(warning)
        self.is_modded = True 

    def on_refresh_rate_changed (self, b):
        self.prefs.refreshRate = self.refreshRateSpin.get_value_as_int()
        self.is_modded = True 

    def clear_modified(self):
        self.is_modded = False

    def is_modified(self):
        return self.is_modded
